@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2016 Oracle Corporation
+ * Copyright (C) 2011-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,16 +24,16 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___VBox_HostService_GuestControlService_h
-#define ___VBox_HostService_GuestControlService_h
+#ifndef VBOX_INCLUDED_HostServices_GuestControlSvc_h
+#define VBOX_INCLUDED_HostServices_GuestControlSvc_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
-#include <VBox/types.h>
-#include <VBox/VMMDev.h>
-#include <VBox/VBoxGuest2.h>
+#include <VBox/VMMDevCoreTypes.h>
+#include <VBox/VBoxGuestCoreTypes.h>
 #include <VBox/hgcmsvc.h>
-#include <VBox/log.h>
 #include <iprt/assert.h>
-#include <iprt/string.h>
 
 /* Everything defined in this file lives in this namespace. */
 namespace guestControl {
@@ -84,205 +84,529 @@ namespace guestControl {
 /**
  * Structure keeping the context of a host callback.
  */
-typedef struct VBoxGuestCtrlHostCbCtx
+typedef struct VBOXGUESTCTRLHOSTCBCTX
 {
-    /** HGCM Function number. */
-    uint32_t uFunction;
+    /** HGCM message number. */
+    uint32_t uMessage;
     /** The context ID. */
     uint32_t uContextID;
     /** Protocol version of this guest session. Might
      *  be 0 if not supported. */
     uint32_t uProtocol;
-
 } VBOXGUESTCTRLHOSTCBCTX, *PVBOXGUESTCTRLHOSTCBCTX;
 
 /**
  * Structure for low level HGCM host callback from
  * the guest. No deep copy. */
-typedef struct VBoxGuestCtrlHostCallback
+typedef struct VBOXGUESTCTRLHOSTCALLBACK
 {
-    VBoxGuestCtrlHostCallback(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
-                                : mParms(cParms), mpaParms(paParms) { }
-
     /** Number of HGCM parameters. */
-    uint32_t mParms;
+    uint32_t         mParms;
     /** Actual HGCM parameters. */
     PVBOXHGCMSVCPARM mpaParms;
-
 } VBOXGUESTCTRLHOSTCALLBACK, *PVBOXGUESTCTRLHOSTCALLBACK;
 
+/** @name Host message destination flags.
+ *
+ * This is ORed into the context ID parameter Main after extending it to 64-bit.
+ *
+ * @internal Host internal.
+ * @{ */
+#define VBOX_GUESTCTRL_DST_ROOT_SVC     RT_BIT_64(63)
+#define VBOX_GUESTCTRL_DST_SESSION      RT_BIT_64(62)
+#define VBOX_GUESTCTRL_DST_BOTH         ( VBOX_GUESTCTRL_DST_ROOT_SVC | VBOX_GUESTCTRL_DST_SESSION )
+/** @} */
+
+
 /**
- * The service functions which are callable by host.
+ * The service messages which are callable by host.
  */
-enum eHostFn
+enum eHostMsg
 {
     /**
      * The host asks the client to cancel all pending waits and exit.
      */
-    HOST_CANCEL_PENDING_WAITS = 0,
+    HOST_MSG_CANCEL_PENDING_WAITS = 0,
     /**
      * The host wants to create a guest session.
      */
-    HOST_SESSION_CREATE = 20,
+    HOST_MSG_SESSION_CREATE = 20,
     /**
      * The host wants to close a guest session.
      */
-    HOST_SESSION_CLOSE = 21,
+    HOST_MSG_SESSION_CLOSE = 21,
     /**
-     * The host wants to execute something in the guest. This can be a command line
-     * or starting a program.
-     ** Note: Legacy (VBox < 4.3) command.
+     * The host wants to execute something in the guest. This can be a command
+     * line or starting a program.
      */
-    HOST_EXEC_CMD = 100,
+    HOST_MSG_EXEC_CMD = 100,
     /**
      * Sends input data for stdin to a running process executed by HOST_EXEC_CMD.
-     ** Note: Legacy (VBox < 4.3) command.
      */
-    HOST_EXEC_SET_INPUT = 101,
+    HOST_MSG_EXEC_SET_INPUT = 101,
     /**
      * Gets the current status of a running process, e.g.
      * new data on stdout/stderr, process terminated etc.
-     ** Note: Legacy (VBox < 4.3) command.
      */
-    HOST_EXEC_GET_OUTPUT = 102,
+    HOST_MSG_EXEC_GET_OUTPUT = 102,
     /**
      * Terminates a running guest process.
      */
-    HOST_EXEC_TERMINATE = 110,
+    HOST_MSG_EXEC_TERMINATE = 110,
     /**
      * Waits for a certain event to happen. This can be an input, output
      * or status event.
      */
-    HOST_EXEC_WAIT_FOR = 120,
+    HOST_MSG_EXEC_WAIT_FOR = 120,
     /**
      * Opens a guest file.
      */
-    HOST_FILE_OPEN = 240,
+    HOST_MSG_FILE_OPEN = 240,
     /**
      * Closes a guest file.
      */
-    HOST_FILE_CLOSE = 241,
+    HOST_MSG_FILE_CLOSE,
     /**
      * Reads from an opened guest file.
      */
-    HOST_FILE_READ = 250,
+    HOST_MSG_FILE_READ = 250,
     /**
-     * Reads from an opened guest file at
-     * a specified offset.
+     * Reads from an opened guest file at a specified offset.
      */
-    HOST_FILE_READ_AT = 251,
+    HOST_MSG_FILE_READ_AT,
     /**
      * Write to an opened guest file.
      */
-    HOST_FILE_WRITE = 260,
+    HOST_MSG_FILE_WRITE = 260,
     /**
-     * Write to an opened guest file at
-     * a specified offset.
+     * Write to an opened guest file at a specified offset.
      */
-    HOST_FILE_WRITE_AT = 261,
+    HOST_MSG_FILE_WRITE_AT,
     /**
      * Changes the read & write position of an opened guest file.
      */
-    HOST_FILE_SEEK = 270,
+    HOST_MSG_FILE_SEEK = 270,
     /**
      * Gets the current file position of an opened guest file.
      */
-    HOST_FILE_TELL = 271,
+    HOST_MSG_FILE_TELL,
+    /**
+     * Changes the file size.
+     */
+    HOST_MSG_FILE_SET_SIZE,
     /**
      * Removes a directory on the guest.
      */
-    HOST_DIR_REMOVE = 320,
+    HOST_MSG_DIR_REMOVE = 320,
     /**
      * Renames a path on the guest.
      */
-    HOST_PATH_RENAME = 330
+    HOST_MSG_PATH_RENAME = 330,
+    /**
+     * Retrieves the user's documents directory.
+     */
+    HOST_MSG_PATH_USER_DOCUMENTS,
+    /**
+     * Retrieves the user's home directory.
+     */
+    HOST_MSG_PATH_USER_HOME,
+
+    /** Blow the type up to 32-bits. */
+    HOST_MSG_32BIT_HACK = 0x7fffffff
 };
 
+
 /**
- * The service functions which are called by guest. The numbers may not change,
- * so we hardcode them.
+ * Translates a guest control host message enum to a string.
+ *
+ * @returns Enum string name.
+ * @param   enmMsg              The message to translate.
  */
-enum eGuestFn
+DECLINLINE(const char *) GstCtrlHostMsgtoStr(enum eHostMsg enmMsg)
 {
-    /**
-     * Guest waits for a new message the host wants to process on the guest side.
+    switch (enmMsg)
+    {
+        RT_CASE_RET_STR(HOST_MSG_CANCEL_PENDING_WAITS);
+        RT_CASE_RET_STR(HOST_MSG_SESSION_CREATE);
+        RT_CASE_RET_STR(HOST_MSG_SESSION_CLOSE);
+        RT_CASE_RET_STR(HOST_MSG_EXEC_CMD);
+        RT_CASE_RET_STR(HOST_MSG_EXEC_SET_INPUT);
+        RT_CASE_RET_STR(HOST_MSG_EXEC_GET_OUTPUT);
+        RT_CASE_RET_STR(HOST_MSG_EXEC_TERMINATE);
+        RT_CASE_RET_STR(HOST_MSG_EXEC_WAIT_FOR);
+        RT_CASE_RET_STR(HOST_MSG_FILE_OPEN);
+        RT_CASE_RET_STR(HOST_MSG_FILE_CLOSE);
+        RT_CASE_RET_STR(HOST_MSG_FILE_READ);
+        RT_CASE_RET_STR(HOST_MSG_FILE_READ_AT);
+        RT_CASE_RET_STR(HOST_MSG_FILE_WRITE);
+        RT_CASE_RET_STR(HOST_MSG_FILE_WRITE_AT);
+        RT_CASE_RET_STR(HOST_MSG_FILE_SEEK);
+        RT_CASE_RET_STR(HOST_MSG_FILE_TELL);
+        RT_CASE_RET_STR(HOST_MSG_FILE_SET_SIZE);
+        RT_CASE_RET_STR(HOST_MSG_DIR_REMOVE);
+        RT_CASE_RET_STR(HOST_MSG_PATH_RENAME);
+        RT_CASE_RET_STR(HOST_MSG_PATH_USER_DOCUMENTS);
+        RT_CASE_RET_STR(HOST_MSG_PATH_USER_HOME);
+        RT_CASE_RET_STR(HOST_MSG_32BIT_HACK);
+    }
+    return "Unknown";
+}
+
+
+/**
+ * The service messages which are callable by the guest.
+ *
+ * @note The message numbers cannot be changed.  Please use the first non-zero
+ *       number that's not in use when adding new messages.
+ *
+ * @note Remember to update service.cpp when adding new messages for Main,
+ *       as it validates all incoming messages before passing them on.
+ */
+enum eGuestMsg
+{
+    /** Guest waits for a new message the host wants to process on the guest side.
      * This is a blocking call and can be deferred.
+     *
+     * @note This message is rather odd.  The above description isn't really
+     *       correct.  Yes, it (1) waits for a new message and will return the
+     *       mesage number and parameter count when one is available.   However, it
+     *       is also (2) used to retrieve the message parameters.   For some weird
+     *       reasons it was decided that it should always return VERR_TOO_MUCH_DATA
+     *       when used in the first capacity.
+     *
+     * @note Has a problem if the guest kernel module cancels the HGCM call, as the
+     *       guest cannot resume waiting till the host issues a message for it and
+     *       the cancelled call returns.  The new message may potentially end up in
+     *       /dev/null depending and hang the message conversation between the guest
+     *       and the host (SIGCHLD).
+     *
+     * @deprecated Replaced by GUEST_MSG_PEEK_WAIT, GUEST_MSG_GET and
+     *             GUEST_MSG_CANCEL.
      */
     GUEST_MSG_WAIT = 1,
-    /**
-     * Guest asks the host to cancel all pending waits the guest itself waits on.
-     * This becomes necessary when the guest wants to quit but still waits for
-     * commands from the host.
+    /** Cancels pending calls for this client session.
+     *
+     * This should be used if a GUEST_MSG_PEEK_WAIT or GUEST_MSG_WAIT call gets
+     * interrupted on the client end, so as to prevent being rebuffed with
+     * VERR_RESOURCE_BUSY when restarting the call.
+     *
+     * @retval  VINF_SUCCESS if cancelled any calls.
+     * @retval  VWRN_NOT_FOUND if no callers.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @since   6.0
      */
-    GUEST_CANCEL_PENDING_WAITS = 2,
-    /**
-     * Guest disconnected (terminated normally or due to a crash HGCM
+    GUEST_MSG_CANCEL = 2,
+    /** Guest disconnected (terminated normally or due to a crash HGCM
      * detected when calling service::clientDisconnect().
+     *
+     * @note This is a host side notification message that has no business in this
+     *       enum.  The guest cannot use this message number, host will reject it.
      */
-    GUEST_DISCONNECTED = 3,
-    /**
-     * Sets a message filter to only get messages which have a certain
+    GUEST_MSG_DISCONNECTED = 3,
+    /** Sets a message filter to only get messages which have a certain
      * context ID scheme (that is, a specific session, object etc).
      * Since VBox 4.3+.
+     * @deprecated  Replaced by GUEST_SESSION_ACCEPT.
      */
     GUEST_MSG_FILTER_SET = 4,
-    /**
-     * Unsets (and resets) a previously set message filter.
+    /** Unsets (and resets) a previously set message filter.
+     * @retval  VERR_NOT_IMPLEMENTED since 6.0.
+     * @deprecated  Never needed or used,
      */
     GUEST_MSG_FILTER_UNSET = 5,
+    /** Peeks at the next message, returning immediately.
+     *
+     * Returns two 32-bit parameters, first is the message ID and the second the
+     * parameter count.  May optionally return additional 32-bit parameters with the
+     * sizes of respective message parameters.  To distinguish buffer sizes from
+     * integer parameters, the latter gets their sizes inverted (uint32_t is ~4U,
+     * uint64_t is ~8U).
+     *
+     * Does also support the VM restore checking as in GUEST_MSG_PEEK_WAIT (64-bit
+     * param \# 0), see documentation there.
+     *
+     * @retval  VINF_SUCCESS if a message was pending and is being returned.
+     * @retval  VERR_TRY_AGAIN if no message pending.
+     * @retval  VERR_VM_RESTORED if first parameter is a non-zero 64-bit value that
+     *          does not match VbglR3GetSessionId() any more.  The new value is
+     *          returned.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0
+     */
+    GUEST_MSG_PEEK_NOWAIT = 6,
+    /** Peeks at the next message, waiting for one to arrive.
+     *
+     * Returns two 32-bit parameters, first is the message ID and the second the
+     * parameter count.  May optionally return additional 32-bit parameters with the
+     * sizes of respective message parameters.  To distinguish buffer sizes from
+     * integer parameters, the latter gets their sizes inverted (uint32_t is ~4U,
+     * uint64_t is ~8U).
+     *
+     * To facilitate VM restore checking, the first parameter can be a 64-bit
+     * integer holding the VbglR3GetSessionId() value the guest knowns.  The
+     * function will then check this before going to sleep and return
+     * VERR_VM_RESTORED if it doesn't match, same thing happens when the VM is
+     * restored.
+     *
+     * @retval  VINF_SUCCESS if info about an pending message is being returned.
+     * @retval  VINF_TRY_AGAIN and message set to HOST_CANCEL_PENDING_WAITS if
+     *          cancelled by GUEST_MSG_CANCEL.
+     * @retval  VERR_RESOURCE_BUSY if another thread already made a waiting call.
+     * @retval  VERR_VM_RESTORED if first parameter is a non-zero 64-bit value that
+     *          does not match VbglR3GetSessionId() any more.  The new value is
+     *          returned.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @note    This replaces GUEST_MSG_WAIT.
+     * @since   6.0
+     */
+    GUEST_MSG_PEEK_WAIT = 7,
+    /** Gets the next message, returning immediately.
+     *
+     * All parameters are specific to the message being retrieved, however if the
+     * first one is an integer value it shall be an input parameter holding the
+     * ID of the message being retrieved.  While it would be nice to add a separate
+     * parameter for this purpose, this is difficult without breaking GUEST_MSG_WAIT
+     * compatibility.
+     *
+     * @retval  VINF_SUCCESS if message retrieved and removed from the pending queue.
+     * @retval  VERR_TRY_AGAIN if no message pending.
+     * @retval  VERR_MISMATCH if the incoming message ID does not match the pending.
+     * @retval  VERR_BUFFER_OVERFLOW if a parmeter buffer is too small.  The buffer
+     *          size was updated to reflect the required size.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @note    This replaces GUEST_MSG_WAIT.
+     * @since   6.0
+     */
+    GUEST_MSG_GET = 8,
+    /** Skip message.
+     *
+     * This skips the current message, replying to the main backend as best it can.
+     * Takes between zero and two parameters.  The first parameter is the 32-bit
+     * VBox status code to pass onto Main when skipping the message, defaults to
+     * VERR_NOT_SUPPORTED.  The second parameter is the 32-bit message ID of the
+     * message to skip, by default whatever is first in the queue is removed.  This
+     * is also the case if UINT32_MAX is specified.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_NOT_FOUND if no message pending.
+     * @retval  VERR_MISMATCH if the specified message ID didn't match.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @since   6.0
+     */
+    GUEST_MSG_SKIP = 9,
     /**
      * Skips the current assigned message returned by GUEST_MSG_WAIT.
      * Needed for telling the host service to not keep stale
-     * host commands in the queue.
+     * host messages in the queue.
+     * @deprecated  Replaced by GUEST_MSG_SKIP.
      */
-    GUEST_MSG_SKIP = 10,
-    /**
-     * General reply to a host message. Only contains basic data
-     * along with a simple payload.
+    GUEST_MSG_SKIP_OLD = 10,
+    /** General reply to a host message.
+     * Only contains basic data along with a simple payload.
+     * @todo proper docs.
      */
     GUEST_MSG_REPLY = 11,
-    /**
-     * General message for updating a pending progress for
-     * a long task.
+    /** General message for updating a pending progress for a long task.
+     * @todo proper docs.
      */
     GUEST_MSG_PROGRESS_UPDATE = 12,
+    /** Sets the caller as the master.
+     *
+     * Called by the root VBoxService to explicitly tell the host that's the master
+     * service.  Required to use main VBoxGuest device node.  No parameters.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_ACCESS_DENIED if not using main VBoxGuest device not
+     * @retval  VERR_RESOURCE_BUSY if there is already a master.
+     * @retval  VERR_VERSION_MISMATCH if VBoxGuest didn't supply requestor info.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @since   6.0
+     */
+    GUEST_MSG_MAKE_ME_MASTER = 13,
+    /** Prepares the starting of a session.
+     *
+     * VBoxService makes this call before spawning a session process (must be
+     * master). The first parameter is the session ID and the second is a one time
+     * key for identifying the right session process.  First parameter is a 32-bit
+     * session ID with a value between 1 and 0xfff0.  The second parameter is a byte
+     * buffer containing a key that GUEST_SESSION_ACCEPT checks against, minimum
+     * length is 64 bytes, maximum 16384 bytes.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_OUT_OF_RESOURCES if too many pending sessions hanging around.
+     * @retval  VERR_OUT_OF_RANGE if the session ID outside the allowed range.
+     * @retval  VERR_BUFFER_OVERFLOW if key too large.
+     * @retval  VERR_BUFFER_UNDERFLOW if key too small.
+     * @retval  VERR_ACCESS_DENIED if not master or in legacy mode.
+     * @retval  VERR_DUPLICATE if the session ID has been prepared already.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0
+     */
+    GUEST_MSG_SESSION_PREPARE = 14,
+    /** Cancels a prepared session.
+     *
+     * VBoxService makes this call to clean up after spawning a session process
+     * failed.  One parameter, 32-bit session ID.  If UINT32_MAX is passed, all
+     * prepared sessions are cancelled.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VWRN_NOT_FOUND if no session with the specified ID.
+     * @retval  VERR_ACCESS_DENIED if not master or in legacy mode.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0
+     */
+    GUEST_MSG_SESSION_CANCEL_PREPARED = 15,
+    /** Accepts a prepared session.
+     *
+     * The session processes makes this call to accept a prepared session.  The
+     * session ID is then uniquely associated with the HGCM client ID of the caller.
+     * The parameters must be identical to the matching GUEST_SESSION_PREPARE call.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_NOT_FOUND if the specified session ID wasn't found.
+     * @retval  VERR_OUT_OF_RANGE if the session ID outside the allowed range.
+     * @retval  VERR_BUFFER_OVERFLOW if key too large.
+     * @retval  VERR_BUFFER_UNDERFLOW if key too small.
+     * @retval  VERR_ACCESS_DENIED if we're in legacy mode or is master.
+     * @retval  VERR_RESOURCE_BUSY if the client is already associated with a session.
+     * @retval  VERR_MISMATCH if the key didn't match.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0
+     */
+    GUEST_MSG_SESSION_ACCEPT = 16,
     /**
      * Guest reports back a guest session status.
+     * @todo proper docs.
      */
-    GUEST_SESSION_NOTIFY = 20,
+    GUEST_MSG_SESSION_NOTIFY = 20,
     /**
      * Guest wants to close a specific guest session.
+     * @todo proper docs.
      */
-    GUEST_SESSION_CLOSE = 21,
+    GUEST_MSG_SESSION_CLOSE = 21,
+
+    /** Report guest side feature flags and retrieve the host ones.
+     *
+     * VBoxService makes this call right after becoming master to indicate to the
+     * host what features it support in addition.  In return the host will return
+     * features the host supports.  Two 64-bit parameters are passed in from the
+     * guest with the guest features (VBOX_GUESTCTRL_GF_XXX), the host replies by
+     * replacing the parameter values with the host ones (VBOX_GUESTCTRL_HF_XXX).
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_ACCESS_DENIED it not master.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0.10, 5.2.32
+     */
+    GUEST_MSG_REPORT_FEATURES,
+    /** Query the host ones feature masks.
+     *
+     * This is for the session sub-process so that it can get hold of the features
+     * from the host.  Again, it is prudent to set the 127 bit and observe it being
+     * cleared on success, as older hosts might return success without doing
+     * anything.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0.10, 5.2.32
+     */
+    GUEST_MSG_QUERY_FEATURES,
+
     /**
      * Guests sends output from an executed process.
+     * @todo proper docs.
      */
-    GUEST_EXEC_OUTPUT = 100,
+    GUEST_MSG_EXEC_OUTPUT = 100,
     /**
      * Guest sends a status update of an executed process to the host.
+     * @todo proper docs.
      */
-    GUEST_EXEC_STATUS = 101,
+    GUEST_MSG_EXEC_STATUS = 101,
     /**
      * Guests sends an input status notification to the host.
+     * @todo proper docs.
      */
-    GUEST_EXEC_INPUT_STATUS = 102,
+    GUEST_MSG_EXEC_INPUT_STATUS = 102,
     /**
      * Guest notifies the host about some I/O event. This can be
      * a stdout, stderr or a stdin event. The actual event only tells
      * how many data is available / can be sent without actually
      * transmitting the data.
+     * @todo proper docs.
      */
-    GUEST_EXEC_IO_NOTIFY = 210,
+    GUEST_MSG_EXEC_IO_NOTIFY = 210,
     /**
      * Guest notifies the host about some directory event.
+     * @todo proper docs.
      */
-    GUEST_DIR_NOTIFY = 230,
+    GUEST_MSG_DIR_NOTIFY = 230,
     /**
      * Guest notifies the host about some file event.
+     * @todo proper docs.
      */
-    GUEST_FILE_NOTIFY = 240
+    GUEST_MSG_FILE_NOTIFY = 240
 };
 
+/**
+ * Translates a guest control guest message enum to a string.
+ *
+ * @returns Enum string name.
+ * @param   enmMsg              The message to translate.
+ */
+DECLINLINE(const char *) GstCtrlGuestMsgToStr(enum eGuestMsg enmMsg)
+{
+    switch (enmMsg)
+    {
+        RT_CASE_RET_STR(GUEST_MSG_WAIT);
+        RT_CASE_RET_STR(GUEST_MSG_CANCEL);
+        RT_CASE_RET_STR(GUEST_MSG_DISCONNECTED);
+        RT_CASE_RET_STR(GUEST_MSG_FILTER_SET);
+        RT_CASE_RET_STR(GUEST_MSG_FILTER_UNSET);
+        RT_CASE_RET_STR(GUEST_MSG_PEEK_NOWAIT);
+        RT_CASE_RET_STR(GUEST_MSG_PEEK_WAIT);
+        RT_CASE_RET_STR(GUEST_MSG_GET);
+        RT_CASE_RET_STR(GUEST_MSG_SKIP_OLD);
+        RT_CASE_RET_STR(GUEST_MSG_REPLY);
+        RT_CASE_RET_STR(GUEST_MSG_PROGRESS_UPDATE);
+        RT_CASE_RET_STR(GUEST_MSG_SKIP);
+        RT_CASE_RET_STR(GUEST_MSG_MAKE_ME_MASTER);
+        RT_CASE_RET_STR(GUEST_MSG_SESSION_PREPARE);
+        RT_CASE_RET_STR(GUEST_MSG_SESSION_CANCEL_PREPARED);
+        RT_CASE_RET_STR(GUEST_MSG_SESSION_ACCEPT);
+        RT_CASE_RET_STR(GUEST_MSG_SESSION_NOTIFY);
+        RT_CASE_RET_STR(GUEST_MSG_SESSION_CLOSE);
+        RT_CASE_RET_STR(GUEST_MSG_REPORT_FEATURES);
+        RT_CASE_RET_STR(GUEST_MSG_QUERY_FEATURES);
+        RT_CASE_RET_STR(GUEST_MSG_EXEC_OUTPUT);
+        RT_CASE_RET_STR(GUEST_MSG_EXEC_STATUS);
+        RT_CASE_RET_STR(GUEST_MSG_EXEC_INPUT_STATUS);
+        RT_CASE_RET_STR(GUEST_MSG_EXEC_IO_NOTIFY);
+        RT_CASE_RET_STR(GUEST_MSG_DIR_NOTIFY);
+        RT_CASE_RET_STR(GUEST_MSG_FILE_NOTIFY);
+    }
+    return "Unknown";
+}
+
+
 /**
  * Guest session notification types.
  * @sa HGCMMsgSessionNotify.
@@ -340,14 +664,18 @@ enum GUEST_FILE_NOTIFYTYPE
     GUEST_FILE_NOTIFYTYPE_OPEN = 10,
     GUEST_FILE_NOTIFYTYPE_CLOSE = 20,
     GUEST_FILE_NOTIFYTYPE_READ = 30,
+    GUEST_FILE_NOTIFYTYPE_READ_OFFSET,  /**< @since 6.0.10, 5.2.32 - VBOX_GUESTCTRL_HF_0_NOTIFY_RDWR_OFFSET */
     GUEST_FILE_NOTIFYTYPE_WRITE = 40,
+    GUEST_FILE_NOTIFYTYPE_WRITE_OFFSET, /**< @since 6.0.10, 5.2.32 - VBOX_GUESTCTRL_HF_0_NOTIFY_RDWR_OFFSET */
     GUEST_FILE_NOTIFYTYPE_SEEK = 50,
-    GUEST_FILE_NOTIFYTYPE_TELL = 60
+    GUEST_FILE_NOTIFYTYPE_TELL = 60,
+    GUEST_FILE_NOTIFYTYPE_SET_SIZE
 };
 
 /**
- * Guest file seeking types. Has to
- * match FileSeekType in Main.
+ * Guest file seeking types. Has to match FileSeekType in Main.
+ *
+ * @note This is not compatible with RTFileSeek, which is an unncessary pain.
  */
 enum GUEST_FILE_SEEKTYPE
 {
@@ -356,39 +684,61 @@ enum GUEST_FILE_SEEKTYPE
     GUEST_FILE_SEEKTYPE_END = 8
 };
 
+/** @name VBOX_GUESTCTRL_GF_XXX - Guest features.
+ * @sa GUEST_MSG_REPORT_FEATURES
+ * @{ */
+/** Supports HOST_MSG_FILE_SET_SIZE. */
+#define VBOX_GUESTCTRL_GF_0_SET_SIZE                RT_BIT_64(0)
+/** Supports (fixes) treating argv[0] separately from the actual execution command.
+ *  Without this flag the actual execution command is taken as argv[0]. */
+#define VBOX_GUESTCTRL_GF_0_PROCESS_ARGV0           RT_BIT_64(1)
+/** Bit that must be set in the 2nd parameter, will be cleared if the host reponds
+ * correctly (old hosts might not). */
+#define VBOX_GUESTCTRL_GF_1_MUST_BE_ONE             RT_BIT_64(63)
+/** @} */
+
+/** @name VBOX_GUESTCTRL_HF_XXX - Host features.
+ * @sa GUEST_MSG_REPORT_FEATURES
+ * @{ */
+/** Host supports the GUEST_FILE_NOTIFYTYPE_READ_OFFSET and
+ *  GUEST_FILE_NOTIFYTYPE_WRITE_OFFSET notification types. */
+#define VBOX_GUESTCTRL_HF_0_NOTIFY_RDWR_OFFSET      RT_BIT_64(0)
+/** Host supports sending (treating) argv[0] separately from the actual execution command.
+ *  Needed when newer Guest Additions which support VBOX_GUESTCTRL_GF_0_PROCESS_ARGV0 run on an older
+ *  host which doesn't in turn support VBOX_GUESTCTRL_HF_0_PROCESS_ARGV0. */
+#define VBOX_GUESTCTRL_HF_0_PROCESS_ARGV0           RT_BIT_64(1)
+/** @} */
+
+
 /*
  * HGCM parameter structures.
  */
 #pragma pack (1)
 
 /**
- * Waits for a host command to arrive. The structure then contains the
+ * Waits for a host message to arrive. The structure then contains the
  * actual message type + required number of parameters needed to successfully
- * retrieve that host command (in a next round).
+ * retrieve that host message (in a next round).
  */
-typedef struct HGCMMsgCmdWaitFor
+typedef struct HGCMMsgWaitFor
 {
-    VBoxGuestHGCMCallInfo hdr;
-    /**
-     * The returned command the host wants to
-     * run on the guest.
-     */
+    VBGLIOCHGCMCALL hdr;
+    /** The returned message the host wants to run on the guest. */
     HGCMFunctionParameter msg;       /* OUT uint32_t */
     /** Number of parameters the message needs. */
     HGCMFunctionParameter num_parms; /* OUT uint32_t */
-} HGCMMsgCmdWaitFor;
+} HGCMMsgWaitFor;
 
 /**
- * Asks the guest control host service to set a command
+ * Asks the guest control host service to set a message
  * filter for this client. This filter will then only
  * deliver messages to the client which match the
  * wanted context ID (ranges).
  */
-typedef struct HGCMMsgCmdFilterSet
+typedef struct HGCMMsgFilterSet
 {
-    VBoxGuestHGCMCallInfo hdr;
-    /** Value to filter for after filter mask
-     *  was applied. */
+    VBGLIOCHGCMCALL hdr;
+    /** Value to filter for after filter mask was applied. */
     HGCMFunctionParameter value;         /* IN uint32_t */
     /** Mask to add to the current set filter. */
     HGCMFunctionParameter mask_add;      /* IN uint32_t */
@@ -396,30 +746,30 @@ typedef struct HGCMMsgCmdFilterSet
     HGCMFunctionParameter mask_remove;   /* IN uint32_t */
     /** Filter flags; currently unused. */
     HGCMFunctionParameter flags;         /* IN uint32_t */
-} HGCMMsgCmdFilterSet;
+} HGCMMsgFilterSet;
 
 /**
  * Asks the guest control host service to disable
  * a previously set message filter again.
  */
-typedef struct HGCMMsgCmdFilterUnset
+typedef struct HGCMMsgFilterUnset
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Unset flags; currently unused. */
     HGCMFunctionParameter flags;    /* IN uint32_t */
-} HGCMMsgCmdFilterUnset;
+} HGCMMsgFilterUnset;
 
 /**
  * Asks the guest control host service to skip the
- * currently assigned host command returned by
+ * currently assigned host message returned by
  * VbglR3GuestCtrlMsgWaitFor().
  */
-typedef struct HGCMMsgCmdSkip
+typedef struct HGCMMsgSkip
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Skip flags; currently unused. */
     HGCMFunctionParameter flags;    /* IN uint32_t */
-} HGCMMsgCmdSkip;
+} HGCMMsgSkip;
 
 /**
  * Asks the guest control host service to cancel all pending (outstanding)
@@ -427,12 +777,12 @@ typedef struct HGCMMsgCmdSkip
  */
 typedef struct HGCMMsgCancelPendingWaits
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
 } HGCMMsgCancelPendingWaits;
 
-typedef struct HGCMMsgCmdReply
+typedef struct HGCMMsgReply
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Message type. */
@@ -441,14 +791,14 @@ typedef struct HGCMMsgCmdReply
     HGCMFunctionParameter rc;
     /** Optional payload to this reply. */
     HGCMFunctionParameter payload;
-} HGCMMsgCmdReply;
+} HGCMMsgReply;
 
 /**
  * Creates a guest session.
  */
 typedef struct HGCMMsgSessionOpen
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The guest control protocol version this
@@ -469,7 +819,7 @@ typedef struct HGCMMsgSessionOpen
  */
 typedef struct HGCMMsgSessionClose
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Session termination flags. */
@@ -481,7 +831,7 @@ typedef struct HGCMMsgSessionClose
  */
 typedef struct HGCMMsgSessionNotify
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Notification type. */
@@ -492,7 +842,7 @@ typedef struct HGCMMsgSessionNotify
 
 typedef struct HGCMMsgPathRename
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** UInt32: Context ID. */
     HGCMFunctionParameter context;
     /** Source to rename. */
@@ -503,12 +853,26 @@ typedef struct HGCMMsgPathRename
     HGCMFunctionParameter flags;
 } HGCMMsgPathRename;
 
+typedef struct HGCMMsgPathUserDocuments
+{
+    VBGLIOCHGCMCALL hdr;
+    /** UInt32: Context ID. */
+    HGCMFunctionParameter context;
+} HGCMMsgPathUserDocuments;
+
+typedef struct HGCMMsgPathUserHome
+{
+    VBGLIOCHGCMCALL hdr;
+    /** UInt32: Context ID. */
+    HGCMFunctionParameter context;
+} HGCMMsgPathUserHome;
+
 /**
  * Executes a command inside the guest.
  */
 typedef struct HGCMMsgProcExec
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The command to execute on the guest. */
@@ -563,7 +927,7 @@ typedef struct HGCMMsgProcExec
  */
 typedef struct HGCMMsgProcInput
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID) to send the input to. */
@@ -582,7 +946,7 @@ typedef struct HGCMMsgProcInput
  */
 typedef struct HGCMMsgProcOutput
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -600,7 +964,7 @@ typedef struct HGCMMsgProcOutput
  */
 typedef struct HGCMMsgProcStatus
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -618,7 +982,7 @@ typedef struct HGCMMsgProcStatus
  */
 typedef struct HGCMMsgProcStatusInput
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -640,7 +1004,7 @@ typedef struct HGCMMsgProcStatusInput
  */
 typedef struct HGCMMsgProcTerminate
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -652,7 +1016,7 @@ typedef struct HGCMMsgProcTerminate
  */
 typedef struct HGCMMsgProcWaitFor
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -665,7 +1029,7 @@ typedef struct HGCMMsgProcWaitFor
 
 typedef struct HGCMMsgDirRemove
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** UInt32: Context ID. */
     HGCMFunctionParameter context;
     /** Directory to remove. */
@@ -679,7 +1043,7 @@ typedef struct HGCMMsgDirRemove
  */
 typedef struct HGCMMsgFileOpen
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** UInt32: Context ID. */
     HGCMFunctionParameter context;
     /** File to open. */
@@ -701,7 +1065,7 @@ typedef struct HGCMMsgFileOpen
  */
 typedef struct HGCMMsgFileClose
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to close. */
@@ -713,7 +1077,7 @@ typedef struct HGCMMsgFileClose
  */
 typedef struct HGCMMsgFileRead
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to read from. */
@@ -727,7 +1091,7 @@ typedef struct HGCMMsgFileRead
  */
 typedef struct HGCMMsgFileReadAt
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to read from. */
@@ -743,7 +1107,7 @@ typedef struct HGCMMsgFileReadAt
  */
 typedef struct HGCMMsgFileWrite
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to write to. */
@@ -759,7 +1123,7 @@ typedef struct HGCMMsgFileWrite
  */
 typedef struct HGCMMsgFileWriteAt
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to write to. */
@@ -777,7 +1141,7 @@ typedef struct HGCMMsgFileWriteAt
  */
 typedef struct HGCMMsgFileSeek
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to seek. */
@@ -793,13 +1157,28 @@ typedef struct HGCMMsgFileSeek
  */
 typedef struct HGCMMsgFileTell
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to get the current position for. */
     HGCMFunctionParameter handle;
 } HGCMMsgFileTell;
 
+/**
+ * Changes the file size.
+ */
+typedef struct HGCMMsgFileSetSize
+{
+    VBGLIOCHGCMCALL         Hdr;
+    /** Context ID. */
+    HGCMFunctionParameter   id32Context;
+    /** File handle to seek. */
+    HGCMFunctionParameter   id32Handle;
+    /** The new file size. */
+    HGCMFunctionParameter   cb64NewSize;
+} HGCMMsgFileSetSize;
+
+
 /******************************************************************************
 * HGCM replies from the guest. These are handled in Main's low-level HGCM     *
 * callbacks and dispatched to the appropriate guest object.                   *
@@ -807,7 +1186,7 @@ typedef struct HGCMMsgFileTell
 
 typedef struct HGCMReplyFileNotify
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Notification type. */
@@ -828,11 +1207,25 @@ typedef struct HGCMReplyFileNotify
             HGCMFunctionParameter data;
         } read;
         struct
+        {
+            /** Actual data read (if any). */
+            HGCMFunctionParameter pvData;
+            /** The new file offset (signed).  Negative value if non-seekable files. */
+            HGCMFunctionParameter off64New;
+        } ReadOffset;
+        struct
         {
             /** How much data (in bytes) have been successfully written. */
             HGCMFunctionParameter written;
         } write;
         struct
+        {
+            /** Number of bytes that was successfully written. */
+            HGCMFunctionParameter cb32Written;
+            /** The new file offset (signed).  Negative value if non-seekable files. */
+            HGCMFunctionParameter off64New;
+        } WriteOffset;
+        struct
         {
             HGCMFunctionParameter offset;
         } seek;
@@ -840,12 +1233,16 @@ typedef struct HGCMReplyFileNotify
         {
             HGCMFunctionParameter offset;
         } tell;
+        struct
+        {
+            HGCMFunctionParameter cb64Size;
+        } SetSize;
     } u;
 } HGCMReplyFileNotify;
 
 typedef struct HGCMReplyDirNotify
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Notification type. */
@@ -1054,10 +1451,15 @@ typedef struct CALLBACKDATA_FILE_NOTIFY
             /** New file offset after successful tell. */
             uint64_t uOffActual;
         } tell;
+        struct
+        {
+            /** The new file siz.e */
+            uint64_t cbSize;
+        } SetSize;
     } u;
 } CALLBACKDATA_FILE_NOTIFY, *PCALLBACKDATA_FILE_NOTIFY;
 
 } /* namespace guestControl */
 
-#endif  /* !___VBox_HostService_GuestControlService_h */
+#endif /* !VBOX_INCLUDED_HostServices_GuestControlSvc_h */
 
