@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,8 +23,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___iprt_initterm_h
-#define ___iprt_initterm_h
+#ifndef IPRT_INCLUDED_initterm_h
+#define IPRT_INCLUDED_initterm_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/cdefs.h>
 #include <iprt/types.h>
@@ -46,27 +49,28 @@ RT_C_DECLS_BEGIN
  */
 
 #ifdef IN_RING3
+
 /** @name RTR3Init flags (RTR3INIT_XXX).
  * @{ */
 /** Try initialize SUPLib. */
-#define RTR3INIT_FLAGS_SUPLIB       RT_BIT(0)
+# define RTR3INIT_FLAGS_SUPLIB       RT_BIT(0)
 /** Initializing IPRT from a DLL. */
-#define RTR3INIT_FLAGS_DLL          RT_BIT(1)
+# define RTR3INIT_FLAGS_DLL          RT_BIT(1)
 /** We are sharing a process space, so we need to behave. */
-#define RTR3INIT_FLAGS_UNOBTRUSIVE  RT_BIT(2)
+# define RTR3INIT_FLAGS_UNOBTRUSIVE  RT_BIT(2)
 /** The caller ensures that the argument bector is UTF-8. */
-#define RTR3INIT_FLAGS_UTF8_ARGV    RT_BIT(3)
+# define RTR3INIT_FLAGS_UTF8_ARGV    RT_BIT(3)
 /** Indicates that this is a standalone application without any additional
  * shared libraries in the application directory. Mainly windows loader mess. */
-#define RTR3INIT_FLAGS_STANDALONE_APP RT_BIT(4)
+# define RTR3INIT_FLAGS_STANDALONE_APP RT_BIT(4)
 /** @} */
 
 /** @name RTR3InitEx version
  * @{ */
 /** Version 1. */
-#define RTR3INIT_VER_1              UINT32_C(1)
+# define RTR3INIT_VER_1              UINT32_C(1)
 /** The current version. */
-#define RTR3INIT_VER_CUR            RTR3INIT_VER_1
+# define RTR3INIT_VER_CUR            RTR3INIT_VER_1
 /** @} */
 
 /**
@@ -128,6 +132,7 @@ RTR3DECL(bool) RTR3InitIsInitialized(void);
  * @returns true/false.
  */
 RTR3DECL(bool) RTR3InitIsUnobtrusive(void);
+
 #endif /* IN_RING3 */
 
 
@@ -259,5 +264,5 @@ RTDECL(void) RTTermRunCallbacks(RTTERMREASON enmReason, int32_t iStatus);
 RT_C_DECLS_END
 
 
-#endif
+#endif /* !IPRT_INCLUDED_initterm_h */
 
