@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,8 +23,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___iprt_process_h
-#define ___iprt_process_h
+#ifndef IPRT_INCLUDED_process_h
+#define IPRT_INCLUDED_process_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/cdefs.h>
 #include <iprt/types.h>
@@ -164,6 +167,9 @@ RTR3DECL(int)   RTProcCreate(const char *pszExec, const char * const *papszArgs,
  * @param   pszPassword Password to use to authenticate @a pszAsUser.  Must be
  *                      NULL wif pszAsUser is NULL.  Whether this is actually
  *                      used or not depends on the platform.
+ * @param   pvExtraData Points to additional data as per @a fFlags:
+ *                          - RTPROC_FLAGS_DESIRED_SESSION_ID: Pointing to a
+ *                            uint32_t variable with the desired session ID.
  * @param   phProcess   Where to store the process handle on successful return.
  *                      The content is not changed on failure.  NULL is allowed.
  *
@@ -176,7 +182,7 @@ RTR3DECL(int)   RTProcCreate(const char *pszExec, const char * const *papszArgs,
  */
 RTR3DECL(int)   RTProcCreateEx(const char *pszExec, const char * const *papszArgs, RTENV hEnv, uint32_t fFlags,
                                PCRTHANDLE phStdIn, PCRTHANDLE phStdOut, PCRTHANDLE phStdErr, const char *pszAsUser,
-                               const char *pszPassword, PRTPROCESS phProcess);
+                               const char *pszPassword, void *pvExtraData, PRTPROCESS phProcess);
 
 /** @name RTProcCreate and RTProcCreateEx flags
  * @{ */
@@ -218,8 +224,18 @@ RTR3DECL(int)   RTProcCreateEx(const char *pszExec, const char * const *papszArg
 /** Consider hEnv an environment change record to be applied to RTENV_DEFAULT.
  * If hEnv is RTENV_DEFAULT, the flag has no effect. */
 #define RTPROC_FLAGS_ENV_CHANGE_RECORD      RT_BIT(8)
+/** Create process using the current impersonated thread token.
+ * Caller should also specify RTPROC_FLAGS_SERVICE and RTPROC_FLAGS_PROFILE.
+ * Windows only flag, ignored everywhere else. */
+#define RTPROC_FLAGS_AS_IMPERSONATED_TOKEN  RT_BIT(9)
+/** Hint that we don't expect to ever want to wait on the process. */
+#define RTPROC_FLAGS_NO_WAIT                RT_BIT(10)
+/** For use with RTPROC_FLAGS_SERVICE to specify a desired session ID
+ * (Windows only, ignored elsewhere).  The @a pvExtraData argument points to
+ * a uint32_t containing the session ID, UINT32_MAX means any session. */
+#define RTPROC_FLAGS_DESIRED_SESSION_ID     RT_BIT(11)
 /** Valid flag mask. */
-#define RTPROC_FLAGS_VALID_MASK             UINT32_C(0x1ff)
+#define RTPROC_FLAGS_VALID_MASK             UINT32_C(0xfff)
 /** @}  */
 
 
@@ -417,5 +433,5 @@ RTR3DECL(int)   RTProcQueryUsernameA(RTPROCESS hProcess, char **ppszUser);
 
 RT_C_DECLS_END
 
-#endif
+#endif /* !IPRT_INCLUDED_process_h */
 
