@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -46,11 +46,12 @@
 # include <paths.h>
 #endif
 
-#include <VBox/err.h>
+#include <iprt/errcore.h>
 #include <VBox/vmm/dbgf.h>
 #include <VBox/vmm/pdmdrv.h>
 #include <iprt/assert.h>
 #include <iprt/file.h>
+#include <iprt/path.h>
 #ifndef RT_OS_WINDOWS
 # include <sys/ioctl.h>
 # include <poll.h>
@@ -387,13 +388,18 @@ int slirp_init(PNATState *ppData, uint32_t u32NetAddr, uint32_t u32Netmask,
     inet_aton("127.0.0.1", &loopback_addr);
 
     rc = slirpTftpInit(pData);
-    AssertRCReturn(rc, VINF_NAT_DNS);
+    AssertRCReturn(rc, rc);
 
     if (i32AliasMode & ~(PKT_ALIAS_LOG|PKT_ALIAS_SAME_PORTS|PKT_ALIAS_PROXY_ONLY))
     {
-        Log(("NAT: alias mode %x is ignored\n", i32AliasMode));
+        LogRel(("NAT: bad alias mode 0x%x ignored\n", i32AliasMode));
         i32AliasMode = 0;
     }
+    else if (i32AliasMode != 0)
+    {
+        LogRel(("NAT: alias mode 0x%x\n", i32AliasMode));
+    }
+
     pData->i32AliasMode = i32AliasMode;
     getouraddr(pData);
     {
@@ -573,6 +579,8 @@ void slirp_term(PNATState pData)
 #ifdef RT_OS_WINDOWS
     WSACleanup();
 #endif
+    if (tftp_prefix)
+        RTStrFree((char *)tftp_prefix);
 #ifdef LOG_ENABLED
     Log(("\n"
          "NAT statistics\n"
@@ -680,10 +688,7 @@ void slirp_select_fill(PNATState pData, int *pnfds, struct pollfd *polls)
         so->so_poll_index = -1;
 #endif
         STAM_COUNTER_INC(&pData->StatTCP);
-#ifdef VBOX_WITH_NAT_UDP_SOCKET_CLONE
-        /* TCP socket can't be cloned */
-        Assert((!so->so_cloneOf));
-#endif
+
         /*
          * See if we need a tcp_fasttimo
          */
@@ -792,10 +797,6 @@ void slirp_select_fill(PNATState pData, int *pnfds, struct pollfd *polls)
                 CONTINUE_NO_UNLOCK(udp);
             }
         }
-#ifdef VBOX_WITH_NAT_UDP_SOCKET_CLONE
-        if (so->so_cloneOf)
-                CONTINUE_NO_UNLOCK(udp);
-#endif
 
         /*
          * When UDP packets are received from over the link, they're
@@ -957,10 +958,6 @@ void slirp_select_poll(PNATState pData, struct pollfd *polls, int ndfs)
      */
     QSOCKET_FOREACH(so, so_next, tcp)
     /* { */
-        /* TCP socket can't be cloned */
-#ifdef VBOX_WITH_NAT_UDP_SOCKET_CLONE
-        Assert((!so->so_cloneOf));
-#endif
         Assert(!so->fUnderPolling);
         so->fUnderPolling = 1;
         if (slirpVerifyAndFreeSocket(pData, so))
@@ -1229,10 +1226,6 @@ void slirp_select_poll(PNATState pData, struct pollfd *polls, int ndfs)
      */
      QSOCKET_FOREACH(so, so_next, udp)
      /* { */
-#ifdef VBOX_WITH_NAT_UDP_SOCKET_CLONE
-        if (so->so_cloneOf)
-            CONTINUE_NO_UNLOCK(udp);
-#endif
 #if 0
         so->fUnderPolling = 1;
         if(slirpVerifyAndFreeSocket(pData, so));
@@ -1673,7 +1666,9 @@ void slirp_post_sent(PNATState pData, void *pvArg)
 void slirp_set_dhcp_TFTP_prefix(PNATState pData, const char *tftpPrefix)
 {
     Log2(("tftp_prefix: %s\n", tftpPrefix));
-    tftp_prefix = tftpPrefix;
+    if (tftp_prefix)
+        RTStrFree((char *)tftp_prefix);
+    tftp_prefix = RTPathAbsDup(tftpPrefix);
 }
 
 void slirp_set_dhcp_TFTP_bootfile(PNATState pData, const char *bootFile)
@@ -2033,7 +2028,7 @@ int slirp_host_network_configuration_change_strategy_selector(const PNATState pD
         struct rcp_state rcp_state;
         int rc;
 
-        rcp_state.rcps_flags |= RCPSF_IGNORE_IPV6;
+        rcp_state.rcps_flags = RCPSF_IGNORE_IPV6;
         rc = rcp_parse(&rcp_state, RESOLV_CONF_FILE);
         LogRelFunc(("NAT: rcp_parse:%Rrc old domain:%s new domain:%s\n",
                     rc, LIST_EMPTY(&pData->pDomainList)
