@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2016 Oracle Corporation
+ * Copyright (C) 2011-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -636,10 +636,18 @@ uint16_t ahci_cmd_packet(uint16_t device_id, uint8_t cmdlen, char __far *cmdbuf,
     return ahci->aCmdHdr[1] == 0 ? 4 : 0;
 }
 
+/* Wait for the specified number of BIOS timer ticks or data bytes. */
+void wait_ticks_device_init( unsigned wait_ticks, unsigned wait_bytes )
+{
+}
+
 void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
 {
-    uint32_t            val;
-    bio_dsk_t __far     *bios_dsk;
+    uint32_t                val;
+    bio_dsk_t __far         *bios_dsk;
+    volatile uint32_t __far *ticks;
+    uint32_t                end_tick;
+    int                     device_found = 0;
 
     ahci_port_init(ahci, u8Port);
 
@@ -653,15 +661,31 @@ void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
      */
     VBOXAHCI_PORT_WRITE_REG(ahci->iobase, u8Port, AHCI_REG_PORT_SCTL, 0);
 
-    /* Check if there is a device on the port. */
-    VBOXAHCI_PORT_READ_REG(ahci->iobase, u8Port, AHCI_REG_PORT_SSTS, val);
-    if (ahci_ctrl_extract_bits(val, 0xfL, 0) == 0)
-        return; /* No device detected. */
+    /*
+     * We do however have to wait for the device to initialize (the port reset
+     * to complete). That can take up to 10ms according to the SATA spec (device
+     * must send COMINIT within 10ms of COMRESET). We should be generous with
+     * the wait because in the typical case there are no ports without a device
+     * attached.
+     */
+    ticks = MK_FP( 0x40, 0x6C );
+    end_tick = *ticks + 3;  /* Wait up to five BIOS ticks, something in 150ms range. */
 
-    do
+    while( *ticks < end_tick )
     {
+        /* If PxSSTS.DET is 3, everything went fine. */
         VBOXAHCI_PORT_READ_REG(ahci->iobase, u8Port, AHCI_REG_PORT_SSTS, val);
-    } while (ahci_ctrl_extract_bits(val, 0xfL, 0) == 0x1);
+        if (ahci_ctrl_extract_bits(val, 0xfL, 0) == 3) {
+            device_found = 1;
+            break;
+        }
+    }
+
+    /* Timed out, no device detected. */
+    if (!device_found) {
+        DBG_AHCI("AHCI: Timed out, no device detected on port %d\n", u8Port);
+        return;
+    }
 
     if (ahci_ctrl_extract_bits(val, 0xfL, 0) == 0x3)
     {
@@ -789,11 +813,12 @@ void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
 
                 removable = *(abBuffer+0) & 0x80 ? 1 : 0;
 
-                bios_dsk->ahcidev[devcount_ahci].port = u8Port;
-                bios_dsk->devices[hd_index].type      = DSK_TYPE_AHCI;
-                bios_dsk->devices[hd_index].device    = DSK_DEVICE_CDROM;
-                bios_dsk->devices[hd_index].removable = removable;
-                bios_dsk->devices[hd_index].blksize   = 2048;
+                bios_dsk->ahcidev[devcount_ahci].port   = u8Port;
+                bios_dsk->devices[hd_index].type        = DSK_TYPE_AHCI;
+                bios_dsk->devices[hd_index].device      = DSK_DEVICE_CDROM;
+                bios_dsk->devices[hd_index].removable   = removable;
+                bios_dsk->devices[hd_index].blksize     = 2048;
+                bios_dsk->devices[hd_index].translation = GEO_TRANSLATION_NONE;
 
                 /* Store the ID of the device in the BIOS cdidmap. */
                 cdcount = bios_dsk->cdcount;
@@ -1001,6 +1026,9 @@ void BIOSCALL ahci_init(void)
                         int         rc;
                         uint16_t    u16AhciIoBase = (u32Bar & 0xfff0) + u16Off;
 
+                        /* Enable PCI memory, I/O, bus mastering access in command register. */
+                        pci_write_config_word(u8Bus, u8DevFn, 4, 0x7);
+
                         DBG_AHCI("I/O base: 0x%x\n", u16AhciIoBase);
                         rc = ahci_hba_init(u16AhciIoBase);
                     }
