@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2019 Oracle Corporation
+ * Copyright (C) 2010-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,6 +23,7 @@
 
 /* Qt includes: */
 #include <QObject>
+#include <QRect>
 
 /* COM includes: */
 #include "COMEnums.h"
@@ -33,6 +34,7 @@
 
 /* Forward declarations: */
 class UIConsoleEventHandlerProxy;
+class UIMousePointerShapeData;
 class UISession;
 
 
@@ -44,8 +46,8 @@ class UIConsoleEventHandler : public QObject
 
 signals:
 
-    /** Notifies about mouse pointer become @a fVisible and his shape changed to @a fAlpha, @a hotCorner, @a size and @a shape. */
-    void sigMousePointerShapeChange(bool fVisible, bool fAlpha, QPoint hotCorner, QSize size, QVector<uint8_t> shape);
+    /** Notifies about mouse pointer @a shapeData change. */
+    void sigMousePointerShapeChange(const UIMousePointerShapeData &shapeData);
     /** Notifies about mouse capability change to @a fSupportsAbsolute, @a fSupportsRelative, @a fSupportsMultiTouch and @a fNeedsHostCursor. */
     void sigMouseCapabilityChange(bool fSupportsAbsolute, bool fSupportsRelative, bool fSupportsMultiTouch, bool fNeedsHostCursor);
     /** Notifies about guest request to change the cursor position to @a uX * @a uY.
@@ -85,6 +87,10 @@ signals:
 #endif /* RT_OS_DARWIN */
     /** Notifies about audio adapter state change. */
     void sigAudioAdapterChange();
+    /** Notifies clipboard mode change. */
+    void sigClipboardModeChange(KClipboardMode enmMode);
+    /** Notifies drag and drop mode change. */
+    void sigDnDModeChange(KDnDMode enmMode);
 
 public:
 
@@ -121,4 +127,3 @@ private:
 #define gConsoleEvents UIConsoleEventHandler::instance()
 
 #endif /* !FEQT_INCLUDED_SRC_runtime_UIConsoleEventHandler_h */
-
