@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2014-2016 Oracle Corporation
+ * Copyright (C) 2014-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,6 +19,9 @@
 /*********************************************************************************************************************************
 *   Header Files                                                                                                                 *
 *********************************************************************************************************************************/
+#define LOG_GROUP LOG_GROUP_GUEST_DND //LOG_GROUP_MAIN_GUESTDNDSOURCE
+#include "LoggingNew.h"
+
 #include "GuestImpl.h"
 #include "GuestDnDSourceImpl.h"
 #include "GuestDnDPrivate.h"
@@ -38,11 +41,6 @@
 
 #include <VBox/com/array.h>
 
-#ifdef LOG_GROUP
- #undef LOG_GROUP
-#endif
-#define LOG_GROUP LOG_GROUP_GUEST_DND
-#include <VBox/log.h>
 
 /**
  * Base class for a source task.
@@ -267,10 +265,10 @@ HRESULT GuestDnDSource::dragIsPending(ULONG uScreenId, GuestDnDMIMEList &aFormat
         Msg.setNextUInt32(0); /** @todo ContextID not used yet. */
     Msg.setNextUInt32(uScreenId);
 
-    int rc = GuestDnDInst()->hostCall(Msg.getType(), Msg.getCount(), Msg.getParms());
+    int rc = GUESTDNDINST()->hostCall(Msg.getType(), Msg.getCount(), Msg.getParms());
     if (RT_SUCCESS(rc))
     {
-        GuestDnDResponse *pResp = GuestDnDInst()->response();
+        GuestDnDResponse *pResp = GUESTDNDINST()->response();
         AssertPtr(pResp);
 
         bool fFetchResult = true;
@@ -280,7 +278,7 @@ HRESULT GuestDnDSource::dragIsPending(ULONG uScreenId, GuestDnDMIMEList &aFormat
             fFetchResult = false;
 
         if (   fFetchResult
-            && isDnDIgnoreAction(pResp->defAction()))
+            && isDnDIgnoreAction(pResp->getActionDefault()))
             fFetchResult = false;
 
         /* Fetch the default action to use. */
@@ -300,9 +298,9 @@ HRESULT GuestDnDSource::dragIsPending(ULONG uScreenId, GuestDnDMIMEList &aFormat
                     LogRel3(("DnD:\tFormat #%zu: %s\n", i, lstFiltered.at(i).c_str()));
 
                 aFormats            = lstFiltered;
-                aAllowedActions     = GuestDnD::toMainActions(pResp->allActions());
+                aAllowedActions     = GuestDnD::toMainActions(pResp->getActionsAllowed());
                 if (aDefaultAction)
-                    *aDefaultAction = GuestDnD::toMainAction(pResp->defAction());
+                    *aDefaultAction = GuestDnD::toMainAction(pResp->getActionDefault());
 
                 /* Apply the (filtered) formats list. */
                 m_lstFmtOffered     = lstFiltered;
@@ -311,7 +309,7 @@ HRESULT GuestDnDSource::dragIsPending(ULONG uScreenId, GuestDnDMIMEList &aFormat
                 LogRel2(("DnD: Negotiation of formats between guest and host failed, drag and drop to host not possible\n"));
         }
 
-        LogFlowFunc(("fFetchResult=%RTbool, allActions=0x%x\n", fFetchResult, pResp->allActions()));
+        LogFlowFunc(("fFetchResult=%RTbool, lstActionsAllowed=0x%x\n", fFetchResult, pResp->getActionsAllowed()));
     }
 
     LogFlowFunc(("hr=%Rhrc\n", hr));
@@ -338,8 +336,8 @@ HRESULT GuestDnDSource::drop(const com::Utf8Str &aFormat, DnDAction_T aAction, C
     if (!GuestDnD::isFormatInFormatList(aFormat, m_lstFmtOffered))
         return setError(E_INVALIDARG, tr("Specified format '%s' is not supported"), aFormat.c_str());
 
-    uint32_t uAction = GuestDnD::toHGCMAction(aAction);
-    if (isDnDIgnoreAction(uAction)) /* If there is no usable action, ignore this request. */
+    VBOXDNDACTION dndAction = GuestDnD::toHGCMAction(aAction);
+    if (isDnDIgnoreAction(dndAction)) /* If there is no usable action, ignore this request. */
         return S_OK;
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
@@ -349,7 +347,7 @@ HRESULT GuestDnDSource::drop(const com::Utf8Str &aFormat, DnDAction_T aAction, C
         return setError(E_INVALIDARG, tr("Another drop operation already is in progress"));
 
     /* Dito. */
-    GuestDnDResponse *pResp = GuestDnDInst()->response();
+    GuestDnDResponse *pResp = GUESTDNDINST()->response();
     AssertPtr(pResp);
 
     HRESULT hr = pResp->resetProgress(m_pGuest);
@@ -379,7 +377,7 @@ HRESULT GuestDnDSource::drop(const com::Utf8Str &aFormat, DnDAction_T aAction, C
         /* This function delete pTask in case of exceptions,
          * so there is no need in the call of delete operator. */
         hr = pTask->createThreadWithType(RTTHREADTYPE_MAIN_WORKER);
-
+        pTask = NULL;  /* Note: pTask is now owned by the worker thread. */
     }
     catch (std::bad_alloc &)
     {
@@ -398,10 +396,9 @@ HRESULT GuestDnDSource::drop(const com::Utf8Str &aFormat, DnDAction_T aAction, C
         hr = pResp->queryProgressTo(aProgress.asOutParam());
         ComAssertComRC(hr);
 
-        /* Note: pTask is now owned by the worker thread. */
     }
     else
-        hr = setError(VBOX_E_IPRT_ERROR, tr("Starting thread for GuestDnDSource::i_receiveDataThread failed (%Rhrc)"), hr);
+        hr = setError(hr, tr("Starting thread for GuestDnDSource::i_receiveDataThread failed (%Rhrc)"), hr);
     /* Note: mDataBase.mfTransferIsPending will be set to false again by i_receiveDataThread. */
 
     LogFlowFunc(("Returning hr=%Rhrc\n", hr));
@@ -694,7 +691,7 @@ int GuestDnDSource::i_onReceiveDir(PRECVDATACTX pCtx, const char *pszPath, uint3
 
     GuestDnDURIObjCtx &objCtx = pCtx->mURI.getObj(0); /** @todo Fill in context ID. */
 
-    rc = objCtx.createIntermediate(DnDURIObject::Directory);
+    rc = objCtx.createIntermediate(DnDURIObject::Type_Directory);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -720,7 +717,7 @@ int GuestDnDSource::i_onReceiveDir(PRECVDATACTX pCtx, const char *pszPath, uint3
             AssertRC(rc2);
 
             objCtx.reset();
-            LogRel2(("DnD: Created guest directory on host: %s\n", pszDir));
+            LogRel2(("DnD: Created guest directory '%s' on host\n", pszDir));
         }
         else
             LogRel(("DnD: Error creating guest directory '%s' on host, rc=%Rrc\n", pszDir, rc));
@@ -782,14 +779,14 @@ int GuestDnDSource::i_onReceiveFileHdr(PRECVDATACTX pCtx, const char *pszPath, u
             if (    pObj->IsOpen()
                 && !pObj->IsComplete())
             {
-                AssertMsgFailed(("Object '%s' not complete yet\n", pObj->GetDestPath().c_str()));
+                AssertMsgFailed(("Object '%s' not complete yet\n", pObj->GetDestPathAbs().c_str()));
                 rc = VERR_WRONG_ORDER;
                 break;
             }
 
             if (pObj->IsOpen()) /* File already opened? */
             {
-                AssertMsgFailed(("Current opened object is '%s', close this first\n", pObj->GetDestPath().c_str()));
+                AssertMsgFailed(("Current opened object is '%s', close this first\n", pObj->GetDestPathAbs().c_str()));
                 rc = VERR_WRONG_ORDER;
                 break;
             }
@@ -799,7 +796,7 @@ int GuestDnDSource::i_onReceiveFileHdr(PRECVDATACTX pCtx, const char *pszPath, u
             /*
              * Create new intermediate object to work with.
              */
-            rc = objCtx.createIntermediate();
+            rc = objCtx.createIntermediate(DnDURIObject::Type_File);
         }
 
         if (RT_SUCCESS(rc))
@@ -825,10 +822,10 @@ int GuestDnDSource::i_onReceiveFileHdr(PRECVDATACTX pCtx, const char *pszPath, u
                 break;
             }
 
-            LogFunc(("Rebased to: %s\n", pszPathAbs));
+            LogRel2(("DnD: Absolute file path for guest file on the host is now '%s'\n", pszPathAbs));
 
             /** @todo Add sparse file support based on fFlags? (Use Open(..., fFlags | SPARSE). */
-            rc = pObj->OpenEx(pszPathAbs, DnDURIObject::File, DnDURIObject::Target,
+            rc = pObj->OpenEx(pszPathAbs, DnDURIObject::View_Target,
                               RTFILE_O_CREATE_REPLACE | RTFILE_O_WRITE | RTFILE_O_DENY_WRITE,
                               (fMode & RTFS_UNIX_MASK) | RTFS_UNIX_IRUSR | RTFS_UNIX_IWUSR);
             if (RT_SUCCESS(rc))
@@ -837,6 +834,8 @@ int GuestDnDSource::i_onReceiveFileHdr(PRECVDATACTX pCtx, const char *pszPath, u
                 int rc2 = pCtx->mURI.getDroppedFiles().AddFile(pszPathAbs);
                 AssertRC(rc2);
             }
+            else
+                LogRel(("DnD: Error opening/creating guest file '%s' on host, rc=%Rrc\n", pszPathAbs, rc));
         }
 
         if (RT_SUCCESS(rc))
@@ -845,25 +844,29 @@ int GuestDnDSource::i_onReceiveFileHdr(PRECVDATACTX pCtx, const char *pszPath, u
             if (mDataBase.m_uProtocolVersion >= 2)
                 rc = pObj->SetSize(cbSize);
 
-            /** @todo Unescpae path before printing. */
-            LogRel2(("DnD: Transferring guest file to host: %s (%RU64 bytes, mode 0x%x)\n",
-                     pObj->GetDestPath().c_str(), pObj->GetSize(), pObj->GetMode()));
+            /** @todo Unescape path before printing. */
+            LogRel2(("DnD: Transferring guest file '%s' to host (%RU64 bytes, mode 0x%x)\n",
+                     pObj->GetDestPathAbs().c_str(), pObj->GetSize(), pObj->GetMode()));
 
             /** @todo Set progress object title to current file being transferred? */
 
-            if (!cbSize) /* 0-byte file? Close again. */
-                pObj->Close();
-        }
+            if (pObj->IsComplete()) /* 0-byte file? We're done already. */
+            {
+                /** @todo Sanitize path. */
+                LogRel2(("DnD: Transferring guest file '%s' (0 bytes) to host complete\n", pObj->GetDestPathAbs().c_str()));
 
-        if (RT_FAILURE(rc))
-        {
-            LogRel2(("DnD: Error opening/creating guest file '%s' on host, rc=%Rrc\n",
-                     pObj->GetDestPath().c_str(), rc));
-            break;
+                pCtx->mURI.processObject(*pObj);
+                pObj->Close();
+
+                objCtx.reset();
+            }
         }
 
     } while (0);
 
+    if (RT_FAILURE(rc))
+        LogRel(("DnD: Error receiving guest file header, rc=%Rrc\n", rc));
+
     LogFlowFuncLeaveRC(rc);
     return rc;
 }
@@ -898,14 +901,14 @@ int GuestDnDSource::i_onReceiveFileData(PRECVDATACTX pCtx, const void *pvData, u
 
         if (pObj->IsComplete())
         {
-            LogFlowFunc(("Warning: Object '%s' already completed\n", pObj->GetDestPath().c_str()));
+            LogFlowFunc(("Warning: Object '%s' already completed\n", pObj->GetDestPathAbs().c_str()));
             rc = VERR_WRONG_ORDER;
             break;
         }
 
         if (!pObj->IsOpen()) /* File opened on host? */
         {
-            LogFlowFunc(("Warning: Object '%s' not opened\n", pObj->GetDestPath().c_str()));
+            LogFlowFunc(("Warning: Object '%s' not opened\n", pObj->GetDestPathAbs().c_str()));
             rc = VERR_WRONG_ORDER;
             break;
         }
@@ -924,27 +927,25 @@ int GuestDnDSource::i_onReceiveFileData(PRECVDATACTX pCtx, const void *pvData, u
             if (RT_SUCCESS(rc))
                 rc = updateProgress(&pCtx->mData, pCtx->mpResp, cbWritten);
         }
-        else /* Something went wrong; close the object. */
-            pObj->Close();
+        else
+            LogRel(("DnD: Error writing guest file data for '%s', rc=%Rrc\n", pObj->GetDestPathAbs().c_str(), rc));
 
         if (RT_SUCCESS(rc))
         {
             if (pObj->IsComplete())
             {
                 /** @todo Sanitize path. */
-                LogRel2(("DnD: File transfer to host complete: %s\n", pObj->GetDestPath().c_str()));
+                LogRel2(("DnD: Transferring guest file '%s' to host complete\n", pObj->GetDestPathAbs().c_str()));
                 pCtx->mURI.processObject(*pObj);
                 objCtx.reset();
             }
         }
-        else
-        {
-            /** @todo What to do when the host's disk is full? */
-            LogRel(("DnD: Error writing guest file to host to '%s': %Rrc\n", pObj->GetDestPath().c_str(), rc));
-        }
 
     } while (0);
 
+    if (RT_FAILURE(rc))
+        LogRel(("DnD: Error receiving guest file data, rc=%Rrc\n", rc));
+
     LogFlowFuncLeaveRC(rc);
     return rc;
 }
@@ -958,12 +959,7 @@ int GuestDnDSource::i_receiveData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
 {
     AssertPtrReturn(pCtx, VERR_INVALID_POINTER);
 
-    /* Is this context already in receiving state? */
-    if (ASMAtomicReadBool(&pCtx->mIsActive))
-        return VERR_WRONG_ORDER;
-    ASMAtomicWriteBool(&pCtx->mIsActive, true);
-
-    GuestDnD *pInst = GuestDnDInst();
+    GuestDnD *pInst = GUESTDNDINST();
     if (!pInst)
         return VERR_INVALID_POINTER;
 
@@ -974,6 +970,11 @@ int GuestDnDSource::i_receiveData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
     if (RT_FAILURE(rc))
         return rc;
 
+    /* Is this context already in receiving state? */
+    if (ASMAtomicReadBool(&pCtx->mIsActive))
+        return VERR_WRONG_ORDER;
+    ASMAtomicWriteBool(&pCtx->mIsActive, true);
+
     /*
      * Reset any old data.
      */
@@ -1023,7 +1024,7 @@ int GuestDnDSource::i_receiveData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
         Assert(!pCtx->mFmtRecv.isEmpty());
 
         if (!pCtx->mFmtRecv.equals(pCtx->mFmtReq))
-            LogRel3(("DnD: Requested data in format '%s', receiving in intermediate format '%s' now\n",
+            LogRel2(("DnD: Requested data in format '%s', receiving in intermediate format '%s' now\n",
                      pCtx->mFmtReq.c_str(), pCtx->mFmtRecv.c_str()));
 
         /*
@@ -1041,10 +1042,10 @@ int GuestDnDSource::i_receiveData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
     }
     else /* Just inform the user (if verbose release logging is enabled). */
     {
-        LogRel2(("DnD: The guest does not support format '%s':\n", pCtx->mFmtReq.c_str()));
-        LogRel2(("DnD: Guest offered the following formats:\n"));
+        LogRel(("DnD: The guest does not support format '%s':\n", pCtx->mFmtReq.c_str()));
+        LogRel(("DnD: Guest offered the following formats:\n"));
         for (size_t i = 0; i < pCtx->mFmtOffered.size(); i++)
-            LogRel2(("DnD:\tFormat #%zu: %s\n", i, pCtx->mFmtOffered.at(i).c_str()));
+            LogRel(("DnD:\tFormat #%zu: %s\n", i, pCtx->mFmtOffered.at(i).c_str()));
     }
 
     ASMAtomicWriteBool(&pCtx->mIsActive, false);
@@ -1067,12 +1068,11 @@ void GuestDnDSource::i_receiveDataThreadTask(RecvDataTask *pTask)
         return;
 
     int vrc = pThis->i_receiveData(pTask->getCtx(), RT_INDEFINITE_WAIT /* msTimeout */);
-    AssertRC(vrc);
-/** @todo
- *
- *  r=bird: What happens with @a vrc?
- *
- */
+    if (RT_FAILURE(vrc)) /* In case we missed some error handling within i_receiveData(). */
+    {
+        AssertFailed();
+        LogRel(("DnD: Receiving data from guest failed with %Rrc\n", vrc));
+    }
 
     AutoWriteLock alock(pThis COMMA_LOCKVAL_SRC_POS);
 
@@ -1080,7 +1080,7 @@ void GuestDnDSource::i_receiveDataThreadTask(RecvDataTask *pTask)
     if (pThis->mDataBase.m_cTransfersPending)
         pThis->mDataBase.m_cTransfersPending--;
 
-    LogFlowFunc(("pSource=%p vrc=%Rrc (ignored)\n", (GuestDnDSource *)pThis, vrc));
+    LogFlowFunc(("pSource=%p, vrc=%Rrc (ignored)\n", (GuestDnDSource *)pThis, vrc));
 }
 
 int GuestDnDSource::i_receiveRawData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
@@ -1094,7 +1094,7 @@ int GuestDnDSource::i_receiveRawData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
     GuestDnDResponse *pResp = pCtx->mpResp;
     AssertPtr(pCtx->mpResp);
 
-    GuestDnD *pInst = GuestDnDInst();
+    GuestDnD *pInst = GUESTDNDINST();
     if (!pInst)
         return VERR_INVALID_POINTER;
 
@@ -1161,12 +1161,17 @@ int GuestDnDSource::i_receiveRawData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
 
     if (RT_FAILURE(rc))
     {
-        if (rc == VERR_CANCELLED)
+        if (rc == VERR_CANCELLED) /* Transfer was cancelled by the host. */
         {
-            int rc2 = pCtx->mpResp->setProgress(100, DND_PROGRESS_CANCELLED);
+            /*
+             * Now that we've cleaned up tell the guest side to cancel.
+             * This does not imply we're waiting for the guest to react, as the
+             * host side never must depend on anything from the guest.
+             */
+            int rc2 = sendCancel();
             AssertRC(rc2);
 
-            rc2 = sendCancel();
+            rc2 = pCtx->mpResp->setProgress(100, DND_PROGRESS_CANCELLED);
             AssertRC(rc2);
         }
         else if (rc != VERR_GSTDND_GUEST_ERROR) /* Guest-side error are already handled in the callback. */
@@ -1175,6 +1180,8 @@ int GuestDnDSource::i_receiveRawData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
                                                 rc, GuestDnDSource::i_hostErrorToString(rc));
             AssertRC(rc2);
         }
+
+        rc = VINF_SUCCESS; /* The error was handled by the setProgress() calls above. */
     }
 
     LogFlowFuncLeaveRC(rc);
@@ -1192,7 +1199,7 @@ int GuestDnDSource::i_receiveURIData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
     GuestDnDResponse *pResp = pCtx->mpResp;
     AssertPtr(pCtx->mpResp);
 
-    GuestDnD *pInst = GuestDnDInst();
+    GuestDnD *pInst = GUESTDNDINST();
     if (!pInst)
         return VERR_INVALID_POINTER;
 
@@ -1230,10 +1237,10 @@ int GuestDnDSource::i_receiveURIData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
     {
         rc = droppedFiles.OpenTemp(0 /* fFlags */);
         if (RT_FAILURE(rc))
+        {
+            LogRel(("DnD: Opening dropped files directory '%s' on the host failed with rc=%Rrc\n", droppedFiles.GetDirAbs(), rc));
             break;
-        LogFlowFunc(("rc=%Rrc, strDropDir=%s\n", rc, droppedFiles.GetDirAbs()));
-        if (RT_FAILURE(rc))
-            break;
+        }
 
         /*
          * Receive the URI list.
@@ -1279,28 +1286,32 @@ int GuestDnDSource::i_receiveURIData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout)
 
     if (RT_FAILURE(rc))
     {
+        int rc2 = droppedFiles.Rollback();
+        if (RT_FAILURE(rc2))
+            LogRel(("DnD: Deleting left over temporary files failed (%Rrc), please remove directory '%s' manually\n",
+                    rc2, droppedFiles.GetDirAbs()));
+
         if (rc == VERR_CANCELLED)
         {
-            int rc2 = pCtx->mpResp->setProgress(100, DND_PROGRESS_CANCELLED);
+            /*
+             * Now that we've cleaned up tell the guest side to cancel.
+             * This does not imply we're waiting for the guest to react, as the
+             * host side never must depend on anything from the guest.
+             */
+            rc2 = sendCancel();
             AssertRC(rc2);
 
-            rc2 = sendCancel();
+            rc2 = pCtx->mpResp->setProgress(100, DND_PROGRESS_CANCELLED);
             AssertRC(rc2);
         }
         else if (rc != VERR_GSTDND_GUEST_ERROR) /* Guest-side error are already handled in the callback. */
         {
-            int rc2 = pCtx->mpResp->setProgress(100, DND_PROGRESS_ERROR,
-                                                rc, GuestDnDSource::i_hostErrorToString(rc));
+            rc2 = pCtx->mpResp->setProgress(100, DND_PROGRESS_ERROR,
+                                            rc, GuestDnDSource::i_hostErrorToString(rc));
             AssertRC(rc2);
         }
-    }
 
-    if (RT_FAILURE(rc))
-    {
-        int rc2 = droppedFiles.Rollback();
-        if (RT_FAILURE(rc2))
-            LogRel(("DnD: Deleting left over temporary files failed (%Rrc). Please remove directory manually: %s\n",
-                    rc2, droppedFiles.GetDirAbs()));
+        rc = VINF_SUCCESS; /* The error was handled by the setProgress() calls above. */
     }
 
     droppedFiles.Close();
