@@ -1,12 +1,10 @@
 /* $Id: PerformanceImpl.cpp $ */
-
 /** @file
- *
  * VBox Performance API COM Classes implementation
  */
 
 /*
- * Copyright (C) 2008-2016 Oracle Corporation
+ * Copyright (C) 2008-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -31,10 +29,11 @@
  * 5) Pre-collection methods cannot modify metric data.
  */
 
+#define LOG_GROUP LOG_GROUP_MAIN_PERFORMANCECOLLECTOR
 #include "PerformanceImpl.h"
 
 #include "AutoCaller.h"
-#include "Logging.h"
+#include "LoggingNew.h"
 
 #include <iprt/process.h>
 
@@ -221,8 +220,6 @@ void PerformanceCollector::uninit()
         return;
     }
 
-    mMagic = 0;
-
     /* Destroy unregistered metrics */
     BaseMetricList::iterator it;
     for (it = m.baseMetrics.begin(); it != m.baseMetrics.end();)
@@ -246,6 +243,9 @@ void PerformanceCollector::uninit()
     AssertMsgRC(vrc, ("Failed to destroy resource usage sampling timer (%Rra)\n", vrc));
     m.sampler = NULL;
 
+    /* Invalidate the magic now. */
+    mMagic = 0;
+
     //delete m.factory;
     //m.factory = NULL;
 
@@ -527,8 +527,7 @@ void PerformanceCollector::registerBaseMetric(pm::BaseMetric *baseMetric)
     if (!SUCCEEDED(autoCaller.rc())) return;
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    Log7(("{%p} " LOG_FN_FMT ": obj=%p name=%s\n", this, __PRETTY_FUNCTION__,
-          (void *)baseMetric->getObject(), baseMetric->getName()));
+    Log7Func(("{%p}: obj=%p name=%s\n", this, (void *)baseMetric->getObject(), baseMetric->getName()));
     m.baseMetrics.push_back(baseMetric);
     //LogFlowThisFuncLeave();
 }
@@ -540,7 +539,7 @@ void PerformanceCollector::registerMetric(pm::Metric *metric)
     if (!SUCCEEDED(autoCaller.rc())) return;
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    Log7(("{%p} " LOG_FN_FMT ": obj=%p name=%s\n", this, __PRETTY_FUNCTION__, (void *)metric->getObject(), metric->getName()));
+    Log7Func(("{%p}: obj=%p name=%s\n", this, (void *)metric->getObject(), metric->getName()));
     m.metrics.push_back(metric);
     //LogFlowThisFuncLeave();
 }
@@ -562,8 +561,7 @@ void PerformanceCollector::unregisterBaseMetricsFor(const ComPtr<IUnknown> &aObj
             (*it)->unregister();
             ++n;
         }
-    Log7(("{%p} " LOG_FN_FMT ": obj=%p, name=%s, marked %d metrics\n",
-          this, __PRETTY_FUNCTION__, (void *)aObject, name.c_str(), n));
+    Log7Func(("{%p}: obj=%p, name=%s, marked %d metrics\n", this, (void *)aObject, name.c_str(), n));
     //LogFlowThisFuncLeave();
 }
 
@@ -576,7 +574,7 @@ void PerformanceCollector::unregisterMetricsFor(const ComPtr<IUnknown> &aObject,
     pm::Filter filter(name, aObject);
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    Log7(("{%p} " LOG_FN_FMT ": obj=%p, name=%s\n", this, __PRETTY_FUNCTION__, (void *)aObject, name.c_str()));
+    Log7Func(("{%p}: obj=%p, name=%s\n", this, (void *)aObject, name.c_str()));
     MetricList::iterator it;
     for (it = m.metrics.begin(); it != m.metrics.end();)
         if (filter.match((*it)->getObject(), (*it)->getName()))
@@ -664,7 +662,7 @@ DECLCALLBACK(void) PerformanceCollector::staticSamplerCallback(RTTIMERLR hTimerL
  */
 void PerformanceCollector::samplerCallback(uint64_t iTick)
 {
-    Log4(("{%p} " LOG_FN_FMT ": ENTER\n", this, __PRETTY_FUNCTION__));
+    Log4Func(("{%p}: ENTER\n", this));
     /* No locking until stage 3!*/
 
     pm::CollectorHints hints;
@@ -681,7 +679,7 @@ void PerformanceCollector::samplerCallback(uint64_t iTick)
 
     if (toBeCollected.size() == 0)
     {
-        Log4(("{%p} " LOG_FN_FMT ": LEAVE (nothing to collect)\n", this, __PRETTY_FUNCTION__));
+        Log4Func(("{%p}: LEAVE (nothing to collect)\n", this));
         return;
     }
 
@@ -699,10 +697,10 @@ void PerformanceCollector::samplerCallback(uint64_t iTick)
      * again to see if any base metrics are marked as unregistered.
      * Those should be destroyed now.
      */
-    Log7(("{%p} " LOG_FN_FMT ": before remove_if: toBeCollected.size()=%d\n", this, __PRETTY_FUNCTION__, toBeCollected.size()));
+    Log7Func(("{%p}: before remove_if: toBeCollected.size()=%d\n", this, toBeCollected.size()));
     toBeCollected.remove_if(std::mem_fun(&pm::BaseMetric::isUnregistered));
-    Log7(("{%p} " LOG_FN_FMT ": after remove_if: toBeCollected.size()=%d\n", this, __PRETTY_FUNCTION__, toBeCollected.size()));
-    Log7(("{%p} " LOG_FN_FMT ": before remove_if: m.baseMetrics.size()=%d\n", this, __PRETTY_FUNCTION__, m.baseMetrics.size()));
+    Log7Func(("{%p}: after remove_if: toBeCollected.size()=%d\n", this, toBeCollected.size()));
+    Log7Func(("{%p}: before remove_if: m.baseMetrics.size()=%d\n", this, m.baseMetrics.size()));
     for (it = m.baseMetrics.begin(); it != m.baseMetrics.end();)
         if ((*it)->isUnregistered())
         {
@@ -711,7 +709,7 @@ void PerformanceCollector::samplerCallback(uint64_t iTick)
         }
         else
             ++it;
-    Log7(("{%p} " LOG_FN_FMT ": after remove_if: m.baseMetrics.size()=%d\n", this, __PRETTY_FUNCTION__, m.baseMetrics.size()));
+    Log7Func(("{%p}: after remove_if: m.baseMetrics.size()=%d\n", this, m.baseMetrics.size()));
     /*
      * Now when we have destroyed all base metrics that could
      * try to pull data from unregistered CollectorGuest objects
@@ -722,7 +720,7 @@ void PerformanceCollector::samplerCallback(uint64_t iTick)
     /* Finally, collect the data */
     std::for_each(toBeCollected.begin(), toBeCollected.end(),
                   std::mem_fun(&pm::BaseMetric::collect));
-    Log4(("{%p} " LOG_FN_FMT ": LEAVE\n", this, __PRETTY_FUNCTION__));
+    Log4Func(("{%p}: LEAVE\n", this));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
