@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2016-2017 Oracle Corporation
+ * Copyright (C) 2016-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -29,11 +29,10 @@
 /*********************************************************************************************************************************
 *   Header Files                                                                                                                 *
 *********************************************************************************************************************************/
-
 #include <iprt/asm.h>
-#include <iprt/cdefs.h>
 #include <iprt/ctype.h>
 #include <iprt/dir.h>
+#include <iprt/err.h>
 #include <iprt/env.h>
 #include <iprt/mem.h>
 #include <iprt/path.h>
@@ -41,7 +40,6 @@
 #include <iprt/string.h>
 #include <iprt/symlink.h>
 #include <iprt/thread.h>
-#include <iprt/types.h>
 
 #include <iprt/linux/sysfs.h>
 
